function Yl = SHY(l,theta,phi,mode)

% function Yl = SHY(l,theta,phi)
% theta,phi => row vectors or matrices
% if theta, phi are row vectors of different sizes, then Yl is calculated for
% all possible pairs of (theta,phi), else (same size) only for the corresponding
% pairs.
% This function returns ALL normalized Spherical Harmonics of order l, but for
% m = 0 to l only. For negative m, use:
% Y(l,-m) = (-1)^m*conj(Y(l,m));
% l is a non-negative integer. l can be a vector.
% If mode=1, then all m coeffs are returned.
%
% Sameer Shirdhonkar (7 April 2005)
% Modified: 20 Dec 2006, 31 March 2008				


% legendre returns ALL the Associated Legendre Polynomials of order l
% So size(Pl) = l+1
% Note that this includes the Condon-Shortley phase, and is not normalized

if(min(size(theta)) == 1 && length(theta) ~= length(phi))
  Yl = zeros(length(l), max(l)+1, length(theta), length(phi)); 
else
  if(~all(size(theta)==size(phi)))
    error('SHY:thetaphidim', ...
    'Theta and phi should have the same size, or be 1D vectors.');
  end
  Yl = zeros(length(l), max(l)+1, size(theta,1), size(theta,2)); 
end

for I=1:length(l)
  Pl = legendre(l(I),cos(theta)); 
  % Normalize Pl (for the whole SphericalHarmonic)
  if(min(size(theta)) == 1 && length(theta) ~= length(phi))
    %1D array of same size => use theta ' * phi (matrix mult)
    for m=0:l(I)
      Yl(I,m+1,:,:) = sqrt( (2*l(I)+1)./( 4*pi*prod(l(I)-m+1:l(I)+m) ) ) ...
      .* Pl(m+1,:)' * exp(j*m*phi);
    end
  else		% 2D array => use theta .* phi (elementwise mult)
    if(ndims(Pl)==2)
      Pl = shiftdim(Pl,-1);
    end
 
    for m=0:l(I)
     if(min(size(theta)) == 1 && length(theta) == length(phi) )
       P = squeeze(Pl(:,m+1,:)); 
     else
       P = squeeze(Pl(m+1,:,:)); 
     end
       Yl(I,m+1,:,:) = sqrt( (2*l(I)+1)./( 4*pi*prod(l(I)-m+1:l(I)+m) ) ) ...
      .* P .* exp(j*m*phi);
    end
  end
end

% This doesn't work if l is a vector starting from zero
if(nargin==4 && mode==1)
  Yl=[(-1).^(-l:-1).*conj(Yl(:,end:-1:2)), Yl];
end
