function [err,tmin,al,Ifit] = gensdp(I, N, Ll, model, vp, alpha, alg)

% function [err,tmin,al,Ifit] = gensdp(I, N, Ll, vp)
%
% Input arguments:
% I = image
% N = object model: N{1,2} = normals(theta \in [0,pi], phi \in [-pi,pi] ),
% N{3} = albedo
% Ll = Max order of harmonics to consider - use even number.
% vp = viewpoint
% model = reflectance model of object 	'l' = lambertian'
%				'm' = mirror
%				'p' = phong - model{2} = exponent
%				'ml' = mirror+lambertian
%				'pl' = phong+lambertian - model{2} = exponent
% 
% Output arguments:
% al = SH coeffs of lighting function
% Ifit = image generated by this lighting 				
% err = error returned by SDP
% tmin = normalized error between input and generated images.
%
% Sameer Shirdhonkar (7 April 2005)
% Modified: 20 Dec 2006, 31 Mar 2008				
% Modified: 2 April 2008. 
% Thanks to Johan Lofberg for suggestions on improving the code.

if ~exist('alg','var') || isempty(alg)
  alg = 'sdp';
end

theta = N{1};
phi = N{2};
albedo = N{3};
I = I(:);

global Gaunt;
persistent C_Ll;

if(rem(Ll,2)~=0)
  warning('Ll should be even.');
end
Lt = (Ll+1)^2;
Llhalf = floor(Ll/2);
Lhalft = (Llhalf+1)^2;		% all m in T_L

% Generate basis images from object model
re = 1;		% Real basis
switch model
case 'l',       % Lambertian
  Bl = LamBasis(theta,phi,albedo,Ll,re);
  M = Bl;
case 'm',       % Mirror
  Bm = Basis(2*N{1}+vp(1),N{2}+vp(2),Ll,re);
  M = Bm;
case 'p',       % Phong
  Bp = PhongBasis(N{1},N{2},N{3},model{2},Ll,re);
  M = Bp;
case {'ml','lm'},   % Mirror + Lambertian
  Bl = LamBasis(theta,phi,albedo,Ll,re);
  Bm = Basis(2*N{1}+vp(1),N{2}+vp(2),Ll,re);
  M = alpha*Bm + (1-alpha)*Bl;
case {'pl','lp'},   % Phong + Lambertian
  Bl = LamBasis(theta,phi,albedo,Ll,re);
  Bp = PhongBasis(N{1},N{2},N{3},model{2},Ll,re);
  M = alpha*Bp + (1-alpha)*Bl;
end

save M M;

if(~isempty(vp))
  u = ( sin(theta).*cos(phi) * sin(vp(1))*cos(vp(2)) + sin(theta).*sin(phi) ...
    * sin(vp(1))*sin(vp(2)) + cos(theta)*cos(vp(1)) ) >= 0;
  u = reshape(u,[],1);  
  I = I.*u; 
  M = M.*repmat(u,1,size(M,2));
end

% Remove zero columns, if any (Lambertian/Phong reflection)
cols = find(sum(abs(M)));
M = M(:,cols);

[Qss,Rss] = qr(M,0);
Iss = Qss'*I;

switch alg

case 'lin'

%  if(model == 'l' || model =='p')
%    al = M\I;
%  else
    al = Rss\Iss;
%  end
  al(isnan(al)) = 0;		% remove NaNs
  err = 0;
  
case 'sdp',

  % Create or load the coefficient matrix
  if(isempty(C_Ll) || C_Ll~=Ll)
    C_Ll = Ll;
    load('Gaunt.mat');
    C = zeros(Lhalft,Lhalft,Lt);
 
    for l1=0:Llhalf
      for m1=-l1:l1
        for l2=0:Llhalf
          for m2=-l2:l2
            for l=max(abs(l1-l2),abs(m1-m2)):min(l1+l2,Ll)
              C(l1*(l1+1)+m1+1,l2*(l2+1)+m2+1,l*(l+1)+m1-m2+1) = realGaunt(l,l2,l1,m1-m2,m2,m1);
            end
          end
        end
      end
    end

    save C.mat C;
  else
   % Load pre-calculated C
   load('C.mat');
  end

  % Formulate the SOCP
a = sdpvar(length(cols),1);
t = sdpvar(1,1);

Tl = zeros(Lhalft,Lhalft);
for k=1:length(cols)
  Tl = Tl + (C(1:Lhalft,1:Lhalft,cols(k))+C(1:Lhalft,1:Lhalft,cols(k))')/2*a(k);
end
%Tl = (Tl+Tl')/2;
F = [cone([1-t;Rss*a-Iss],1+t), Tl>0];

assign(a,ones(size(a))*1e-15);  % initialization to small values prevents
% appearence of NaNs in the final solution.

ops = sdpsettings('solver','sdpt3-4','usex0',1,'saveduals',0,'shift',0,...
'cachesolvers',1,'convertconvexquad',0,'verbose',0);

err = solvesdp(F,t,ops);

al = double(a);
%to = double(t);

end

Ifit = M*al;
tmin = norm(I-Ifit)/norm(I);
%[norm(I-Ifit),tmin]
a = zeros(Lt,1);
a(cols) = al;
al = a;
%keyboard;
