function I = render(shl,N2,model,vp,re)

% function I = render(shl,N2,model,re)
%
% Calculates image based on spherical harmonic lighting specified by
% coefficients shl. N2 = {theta,phi,albedo} is the object structure.
% model{1} = 'm' for mirror
% model{1} = 'l' for Lambertian
% model{1} = 'p' (Exponent) for Phong model{2} contains exponent.
% model{1} = 'ml' or 'pl' - mixtures of Lambertian and mirror/Phong - model{3}
% is amount of mirror/Phong
% for mirror and Phong, viewpoint is Z axis
% vp = viewpoint
% re = real or complex harmonics
%
% Sameer Shirdhonkar (7 April 2005)
% Modified: 20 Dec 2006				

if(nargin==3)
  re = 1;		% real basis, else complex basis
end
theta = N2{1};
phi = N2{2};
albedo = N2{3};

if(re>=0)	% render from SH

  L = sqrt(length(shl)) - 1;
  if(L~=floor(L))
    error('size(shl) ~= L^2');
  end
  
  switch model{1}
  case 's',		% Actual light source
    Bs = Basis(N2{1},N2{2},L,re);
    I = Bs*shl';
  case 'm',		% Mirror
    Bm = Basis(2*N2{1}+vp(1),N2{2}+vp(2),L,re);
    I = Bm*shl';
  case 'l',		% Lambertian
    Bl = LamBasis(N2{1},N2{2},N2{3},L,re);
    I = Bl*shl';
  case {'ml','lm'},		% Mixture of mirror and Lambertian
    Bm = Basis(2*N2{1}+vp(1),N2{2}+vp(2),L,re);
    Bl = LamBasis(N2{1},N2{2},N2{3},L,re);
    I = (model{3}*Bm+(1-model{3})*Bl)*shl';
  case 'p',		% Phong
    Bp = PhongBasis(N2{1},N2{2},N2{3},model{2},L,re);
    I = Bp*shl';
  case {'pl','lp'},		% Mixture of Phong and Lambertian
    Bp = PhongBasis(N2{1},N2{2},N2{3},model{2},L,re);
    Bl = LamBasis(N2{1},N2{2},N2{3},L,re);
    I = (model{3}*Bp+(1-model{3})*Bl)*shl';
  end

else	% render from intensities

  x = sin(theta).*cos(phi);
  y = sin(theta).*sin(phi);
  z = cos(theta);
  N3 = [reshape(x,[],1) reshape(y,[],1) reshape(z,[],1)];
  albedo = reshape(albedo,[],1);
  shl = reshape(shl,[],1);
  I = repmat(shl.*albedo,1,length(shl)) .* (N3*N3');
  I = I.*(I>0);
  I = sum(I,1)';

end


% Set `far-side' to zero.

if(~isempty(vp))
  u = ( sin(theta).*cos(phi) * sin(vp(1))*cos(vp(2)) + sin(theta).*sin(phi) ...
    * sin(vp(1))*sin(vp(2)) + cos(theta)*cos(vp(1)) ) >= 0;
  I = I.*reshape(u,length(I),1);
end
